<?php

namespace Drupal\Tests\override_cache_control_header\Unit;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\override_cache_control_header\EventSubscriber\OverrideCacheControlHeader;
use PHPUnit\Framework\TestCase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Unit test for the OverrideCacheControlHeader class.
 *
 * @group override_cache_control_header
 */
class OverrideCacheControlHeaderTest extends TestCase {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $configFactory;

  /**
   * The event subscriber.
   *
   * @var \Drupal\override_cache_control_header\EventSubscriber\OverrideCacheControlHeader
   */
  protected $eventSubscriber;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Mock the config factory.
    $this->configFactory = $this->createMock(ConfigFactoryInterface::class);

    // Create the event subscriber.
    $this->eventSubscriber = new OverrideCacheControlHeader($this->configFactory);
  }

  /**
   * Tests the onViewResponse method.
   */
  public function testOnViewResponse() {
    // Mock the configuration.
    $config = $this->createMock(ImmutableConfig::class);
    $config->expects($this->once())
      ->method('get')
      ->with('urls_header')
      ->willReturn("/test|no-cache");

    $this->configFactory->expects($this->once())
      ->method('get')
      ->with('override_cache_control_header.settings')
      ->willReturn($config);

    // Create a request and response.
    $request = new Request([], [], [], [], [], ['REQUEST_URI' => '/test']);
    $response = new Response();

    // Create the response event.
    $event = new ResponseEvent(
      $this->createMock(HttpKernelInterface::class),
      $request,
      HttpKernelInterface::MASTER_REQUEST,
      $response
    );

    // Call the onViewResponse method.
    $this->eventSubscriber->onViewResponse($event);

    // Assert that the Cache-Control header is set correctly.
    $this->assertStringContainsString('no-cache', $response->headers->get('Cache-Control'));
  }

  /**
   * Tests the getSubscribedEvents method.
   */
  public function testGetSubscribedEvents() {
    $events = OverrideCacheControlHeader::getSubscribedEvents();
    $this->assertArrayHasKey('kernel.response', $events);
    $this->assertEquals(['onViewResponse', 10000], $events['kernel.response'][0]);
  }

}